/** \file DXQueue.h
<h2> Direct3D Triangle Queue - stores triangles before sending them to Direct3D in batches </h2> <br>
Copyright (C) 2011 Xi Chen <br>

-------------------------------------------------------------------------<br>
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version. <br>

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details. <br>

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>. <br>
-------------------------------------------------------------------------<br>

\brief Direct3D Triangle Queue
\author Xi Chen hypernewbie@gmail.com
\date 15/11/2011

*/

#ifndef __DX_QUEUE__
#define __DX_QUEUE__

#include "DXInterface.h"
#include "DXInfo.h"
#include <vector>

using namespace std;

//! Class that manages a single triangle queue
class PolygonQueue {
    //! Handle to vertexbuffer after compilation
    LPDIRECT3DVERTEXBUFFER9 vertexBuffer;
    //! Number of triangles
    int maxPrimCount;
    //! Number of vertices
    int verticeCount;

public:
    //! A list of (uncompiled) vertices
    vector< DXVertex3 > polyQueue;


    PolygonQueue();
    ~PolygonQueue();

    //! Returns number of triangles
    int numPolygons();
    //! Returns number of vertices
    int numVertices();

    //! Releases all associated memory 
    void reset();

    //! Only releases the uncompiled vertice list (vertexbuffer is kept intact)
    //  Recommended to use after compilation, to free up memory.
    void clear();

    //! Pushes a single vertex onto the queue
    void pushVertex(const DXVertex3 *vx);

    //! Pushes a triangle onto the queue
    void pushVertex(const DXVertex3 *v1, const DXVertex3 *v2, const DXVertex3 *v3);

    //! Compile the vertex queue into a Direct3D vertexbuffer
    void compile();

    //! Renders the compiled vertexbuffer
    void render();

};

#endif
