/** \file DXShader.h
<h2> Direct3D Shader handling using the Effects framework </h2> <br>
Copyright (C) 2011 Xi Chen <br>

-------------------------------------------------------------------------<br>
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version. <br>

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details. <br>

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>. <br>
-------------------------------------------------------------------------<br>

\brief Direct3D Shader handler
\author Xi Chen hypernewbie@gmail.com
\date 15/11/2011

*/

#ifndef __DX_SHADER__
#define __DX_SHADER__

#include "DXInterface.h"

#ifndef _RELEASE_
    #define SHADER_DEBUG_DEFAULT false
#else 
    #define SHADER_DEBUG_DEFAULT true
#endif

//! Common shared shader semantics
struct ShaderGlobalSemantics {
    D3DXMATRIX matWorld;
    D3DXMATRIX matView;
    D3DXMATRIX matProj;
    D3DXVECTOR3 cameraPos;
};

//! Class that manages a single shader effect
class Shader {
protected:
    //! Handle to the Direct3D effect
    LPD3DXEFFECT effect;
    //! D3D buffer to store any error messages
    LPD3DXBUFFER errors;
    //! Number of passes in this effect
    UINT numPasses;

public:

    Shader();
    ~Shader();

    //! Loads and compiles a shader from file
    void loadFile(const char* fileName, bool debug = SHADER_DEBUG_DEFAULT, char optimize = 3, bool showMessage = true);

    //! Returns the handle to the Direct3D effect ( mostly used to set effect parameters )
    LPD3DXEFFECT Shader::e();

    //! Set the shared common effect semantics that nearly every shader
    //  needs ( camera position, world transformation...etc )
    void setGlobalSemantics(const ShaderGlobalSemantics *sgs);

    //! Begin application of this shader effect
    int begin(const char *techniqueName);

    //! Returns number of passes in this effect
    int nPasses();

    //! Starts an effect pass given pass index
    void pass(int n);

    //! Ends started effect pass
    void endPass();

    //! Ends application of this shader effect
    void end();

    //! Releases all associated handles
    void release();
};

#endif
