/** \file DXTerrain.h
<h2> Height mapped terrain </h2> <br>
Copyright (C) 2011 Xi Chen <br>

-------------------------------------------------------------------------<br>
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version. <br>

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details. <br>

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>. <br>
-------------------------------------------------------------------------<br>

\brief Height mapped terrain
\author Xi Chen hypernewbie@gmail.com
\date 15/11/2011

*/

#ifndef __DX_TERRAIN__
#define __DX_TERRAIN__

#ifndef NO_TERRAIN
#define NOOGL

#include "DXInterface.h"
#include "DXMatrix.h"
#include <mini/mini.h>
#include <mini/ministub.h>

#define DXTERRAIN_MAX_VERTEX 400000
#define DXTERRAIN_MAX_INDEX 200000
#define DXTERRAIN_HEIGHT_SCALE 10000.0f
#define DXTERRAIN_DEFAULT_DSIZE 1024
#define DXTERRAIN_DEFAULT_DIM 0.0625f

class Terrain {
public:
    // --> Grid Parameters
    int gridDataSize;
    float gridCellAspect;
    float gridDimension;
    float gridHeightScale;
    D3DXVECTOR3 gridCenter;

    // --> Terrain handle
    ministub *miniHandle;

    // --> Terrain geometry
    LPDIRECT3DVERTEXBUFFER9 vertexBuffer;
    LPDIRECT3DINDEXBUFFER9 indexBuffer;
    DXVertex3 *vertexArray;
    short *indexArray;
    int vertexArrayN;
    int indexArrayN;
    int fanCount;
    int fanLastVertex;
    int fanBaseVertex;
    
    // --> Rendering
    bool emptyView;
    float gridSize;

    // --> Height callback function
    float (*elevationfunction)(int i, int j, int S, void *data);

    // --> Functions

    Terrain();
 
    void initalise();

    void release();

    void compile(D3DXVECTOR3 cam, D3DXVECTOR3 view, D3DXVECTOR3 up, float resolution = 1000.0f, float FOV = 60.0f, 
                     float aspect = 1.0f, float nearp = 1.0f, float farp = 100.0f);

    void render();

    int fanAddVertex( D3DXVECTOR3 pos );

    void fanAddIndex( int r, int s, int t );

};

extern Terrain *currentSelectedTerrain;

void dxTerrainRenderBegin();
void dxTerrainRenderVertex(float i,float y,float j);

#endif
#endif