/** \file DXText.h
<h2> Text Rendering Engine </h2> <br>
Copyright (C) 2011 Xi Chen <br>

-------------------------------------------------------------------------<br>
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version. <br>

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details. <br>

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>. <br>
-------------------------------------------------------------------------<br>

\brief Text Rendering Engine
\author Xi Chen hypernewbie@gmail.com
\date 15/11/2011

*/

#ifndef __DX_TEXT__
#define __DX_TEXT__

#include "DXInterface.h"
#include <cstdarg>

#define DXFONT_DEFAULT_RIGHT 10000
#define DXFONT_DEFAULT_BOT 20000
#define DXFONT_DEFAULT_BUFFERSIZE 1024

//! Quick formatted text display function using default font
void dxPrintf( int x, int y, int col, const char *fmt, ...  );

//! Class that manages Direct3D text rendering
class Font {

protected:
    LPD3DXFONT dxFont;
    LPD3DXSPRITE dxFontSprite;

    char *buffer;

public:
    //! Enable printing to sprites to speed up consecutive prints -
    //  Must print between begin() and end() to render the sprite
    bool batch;


    Font();
    ~Font();

    //! Release associated handles
    void release();

    //! Initalise this font
    void create( LPCTSTR fontName = "Arial", int size = 20, char quality = 1, bool italic = false, 
        unsigned int weight = FW_SEMIBOLD,  unsigned int width = 0 );

    //! Print a string
    void print( int x, int y, const char *str, int col = 0xffffffff, int right = DXFONT_DEFAULT_RIGHT, int bot = DXFONT_DEFAULT_BOT );

    //! Print a formatted string
    void printf( int x, int y, int col, const char *fmt, ...  );
 
    //! Begins printing to sprite (batch must be true)
    void begin(DWORD flags = D3DXSPRITE_ALPHABLEND);

    //! Ends printing to sprite, and renders sprite (batch must be true)
    void end();
    
};

#endif
