/** \file DXTexture.h
<h2> Direct3D Texture Manager </h2> <br>
Copyright (C) 2011 Xi Chen <br>

Supports BMP, JPG, TGA, PNG, DDS, PPM, DIB, HDR, PFM

-------------------------------------------------------------------------<br>
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version. <br>

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details. <br>

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>. <br>
-------------------------------------------------------------------------<br>

\brief Texture Manager
\author Xi Chen hypernewbie@gmail.com
\date 15/11/2011

*/

#ifndef __DX_TEXTURE__
#define __DX_TEXTURE__

#include <vector>
#include "DXInterface.h"

using namespace std;

#define TEXFILTER_NONE 0

//! Texture Image structure that stores and manages a texture
struct Texture {
    //! Direct3D texture handle
    LPDIRECT3DTEXTURE9 image;

    Texture();
    ~Texture();
    //! Release this texture
    void release();
    //! Load a texture from file
    void load(const char* fileName, char filter = TEXFILTER_NONE);
    //! Returns the Direct3D texture handle
    operator LPDIRECT3DTEXTURE9();
};

//! Texture Array class that stores and manages multiple textures.
class TextureArray {

private:
    //! Internal texture array
    vector<Texture> textureData;

public:

    ~TextureArray();

    //! Frees up all texture and image data
    void clear();

    //! Loads an image and builds texture
    void load(int texID, const char *fileName, char filter = TEXFILTER_NONE);

    //! The [] operator, so you get go d3dDevice->SetTexture(0, TextureArray[0]);
    LPDIRECT3DTEXTURE9 operator [] (int texID);
};

#endif