/** \file DXInfo.cpp
<h2> Reads & displays system information </h2> <br>
Copyright (C) 2011 Xi Chen <br>

-------------------------------------------------------------------------<br>
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version. <br>

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details. <br>

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>. <br>
-------------------------------------------------------------------------<br>

\brief Reads & displays system information
\author Xi Chen hypernewbie@gmail.com
\date 15/11/2011

*/

#include "../EngineHeader/DXInfo.h"

//! Prints to the console ( wInfo() ) information about the current
//  system, and the system's DirectX capabilities and limitations
void dxDisplayInfo() {
    if( !d3dDevice ) { wError("dxDisplayInfo() invalid device!"); return; }
    D3DCAPS9 c; HRESULT hr = d3dDevice->GetDeviceCaps( &c );
    if(hr != D3D_OK) { wError("dxDisplayInfo() GetDeviceCaps failure!"); return; }
    D3DADAPTER_IDENTIFIER9 adp; hr = d3d->GetAdapterIdentifier( D3DADAPTER_DEFAULT, 0, &adp );
    if(hr != D3D_OK) { wError("dxDisplayInfo() GetAdapterIdentifier failure!"); return; }
    wInfo( "----------== %s ==---------\n", TITLE_TEXT2 );
    wInfo( "Renderer: %s\n", adp.Description);
    wInfo( "Driver: %s [%d.%d.%d.%d]\n", adp.Driver, HIWORD(adp.DriverVersion.HighPart), LOWORD(adp.DriverVersion.HighPart),
        HIWORD(adp.DriverVersion.LowPart), LOWORD(adp.DriverVersion.LowPart) );
    wInfo( "VendorID: %d\n", adp.VendorId);
    wInfo( "DeviceID: %d\n", adp.DeviceId);
    wInfo( "Device: %s\n", adp.DeviceName);
    wInfo( "------------------------------\n");
    wInfo( "Rasterization: %s\n", c.DevCaps|D3DDEVCAPS_HWRASTERIZATION?"Hardware":"Software");
    wInfo( "Transform & Lighting: %s\n", c.DevCaps|D3DDEVCAPS_HWTRANSFORMANDLIGHT?"Hardware":"Software");
    wInfo( "Anisotropic Filtering: %s %dx\n", c.RasterCaps|D3DPRASTERCAPS_ANISOTROPY?"Yes":"No", c.MaxAnisotropy);
    wInfo( "CubeMap: %s\n", c.TextureCaps|D3DPTEXTURECAPS_MIPCUBEMAP?"Yes":"No");
    wInfo( "VolumeMap: %s\n", c.TextureCaps|D3DPTEXTURECAPS_VOLUMEMAP?"Yes":"No");
    wInfo( "MaxTexSize: %d x %d\n", c.MaxTextureWidth, c.MaxTextureHeight);
    wInfo( "MaxPrimCount: %d\n", c.MaxPrimitiveCount );
    wInfo( "MaxVertexIndex: %d\n", c.MaxVertexIndex );
    wInfo( "MaxStreams: %d\n", c.MaxStreams );
    wInfo( "------------------------------\n");
    wInfo( "VertexShader 1.0: %s\n", c.VertexShaderVersion >= D3DVS_VERSION(1,0)?"Yes":"No" );
    wInfo( "VertexShader 2.0: %s\n", c.VertexShaderVersion >= D3DVS_VERSION(2,0)?"Yes":"No" );
    wInfo( "VertexShader 3.0: %s\n", c.VertexShaderVersion >= D3DVS_VERSION(3,0)?"Yes":"No" );
    wInfo( "PixelShader  1.0: %s\n", c.PixelShaderVersion >= D3DPS_VERSION(1,0)?"Yes":"No" );
    wInfo( "PixelShader  2.0: %s\n", c.PixelShaderVersion >= D3DPS_VERSION(2,0)?"Yes":"No" );
    wInfo( "PixelShader  3.0: %s\n", c.PixelShaderVersion >= D3DPS_VERSION(3,0)?"Yes":"No" );
    wInfo( "NumSimultaneousRTs: %d\n", c.NumSimultaneousRTs );
    wInfo( "MaxVertexShader30InstructionSlots: %d\n", c.MaxVertexShader30InstructionSlots );
    wInfo( "MaxPixelShader30InstructionSlots: %d\n", c.MaxPixelShader30InstructionSlots );
}

//! Returns the maximum number of vertices that the current system
//  allows to be sent in a single DrawPrimitive() call.
unsigned int dxMaxPrimCount() {
    D3DCAPS9 c; HRESULT hr = d3dDevice->GetDeviceCaps( &c );
    return c.MaxPrimitiveCount;
}