/** \file DXText.cpp
<h2> Text Rendering Engine </h2> <br>
Copyright (C) 2011 Xi Chen <br>

-------------------------------------------------------------------------<br>
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version. <br>

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details. <br>

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>. <br>
-------------------------------------------------------------------------<br>

\brief Text Rendering Engine
\author Xi Chen hypernewbie@gmail.com
\date 15/11/2011

*/

#include "../EngineHeader/DXText.h"

Font dxDefaultFont;

// --------> Font Methods

Font::Font() { dxFont = NULL; dxFontSprite = NULL; buffer = NULL; batch = true; }

Font::~Font() { release(); }

//! Release associated handles
void Font::release() {
    if(dxFont) dxFont->Release();
    if(dxFontSprite) dxFontSprite->Release();
    if(buffer) delete[] buffer;
}

//! Initalise this font
void Font::create( LPCTSTR fontName , int size, char quality, bool italic, unsigned int weight,  unsigned int width ) {
    release();
    DWORD qual = DEFAULT_QUALITY;
    switch( quality ) {
        case 0: qual = NONANTIALIASED_QUALITY; break;
        case 1: qual = ANTIALIASED_QUALITY; break;
    }
    HRESULT ret = D3DXCreateFont( d3dDevice, size, width, weight, 1, italic, DEFAULT_CHARSET, OUT_DEFAULT_PRECIS, qual, 
        DEFAULT_PITCH || FF_DONTCARE, fontName, &dxFont );
    if( ret != S_OK ) { wError("Font::create() D3DXCreateFont error"); return; }
    ret = D3DXCreateSprite( d3dDevice, &dxFontSprite );
    if( ret != S_OK ) { wError("Font::create() D3DXCreateSprite error"); return; }
    buffer = new char[DXFONT_DEFAULT_BUFFERSIZE];
    if(!buffer) { wError("Font::create() buffer memory error"); return; }
}

//! Print a string
void Font::print( int x, int y, const char *str, int col, int right, int bot ) {
    if(!str) return;
    if(!dxFont||!dxFontSprite) create();
    RECT rct; rct.left = x; rct.top = y; rct.right = right; rct.bottom = bot;
    dxFont->DrawText( batch?dxFontSprite:NULL, (LPCSTR)str, -1, &rct, DT_LEFT, col );
}

//! Print a formatted string
void Font::printf( int x, int y, int col, const char *fmt, ...  ) {
    if(!fmt) return;
    if(!dxFont||!dxFontSprite||!buffer) create();
    va_list varListParser;
    va_start(varListParser, fmt);
      vsprintf_s(buffer, DXFONT_DEFAULT_BUFFERSIZE, fmt, varListParser);
    va_end(varListParser);
    print( x, y, buffer, col );
}

//! Begins printing to sprite (batch must be true)
void Font::begin(DWORD flags) { if(dxFontSprite && batch) dxFontSprite->Begin(flags); }

//! Ends printing to sprite, and renders sprite (batch must be true)
void Font::end() { if(dxFontSprite && batch) dxFontSprite->End(); }

// --------> Functions

//! Quick formatted text display function using default font
void dxPrintf( int x, int y, int col, const char *fmt, ...  ) {
    char *buffer = new char[DXFONT_DEFAULT_BUFFERSIZE];
    if(!buffer) return;
    va_list varListParser;
    va_start(varListParser, fmt);
      vsprintf_s(buffer, DXFONT_DEFAULT_BUFFERSIZE, fmt, varListParser);
    va_end(varListParser);
    dxDefaultFont.batch = false;
    dxDefaultFont.print( x, y, buffer, col );
    delete[] buffer;
}