/** \file Scene.h
<h2> Base parent scene class interface </h2> <br>
Copyright (C) 2011 Xi Chen <br>

-------------------------------------------------------------------------<br>
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version. <br>

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details. <br>

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>. <br>
-------------------------------------------------------------------------<br>

\brief Base scene class interface
\author Xi Chen hypernewbie@gmail.com
\date 15/11/2011

*/

#ifndef __SCENE_H__
#define __SCENE_H__

#include <cstdio>
#include <vector>
#include <algorithm>
#include <d3d9.h>
#include <d3dx9.h>
#include "../EngineHeader/DXInterface.h"
#include "../EngineHeader/DXIntegrate.h"
#include "../EngineHeader/DXCamera.h"
#include "../EngineHeader/DXMatrix.h"
#include "../EngineHeader/DXModel.h"
#include "../EngineHeader/DXPrimitives.h"
#include "../EngineHeader/DXRandom.h"
#include "../EngineHeader/DXTexture.h"
#include "../EngineHeader/DXShader.h"
#include "../EngineHeader/DXQueue.h"

#define SCENE_DEFAULT_FOV 90
#define SCENE_MAX_CAMERAPATHS 32
#define SCENE_DEFAULT_BPM 134.1
#define SCENE_DEFAULT_CAMERAFACTOR 0.8f

using namespace std;

//! Base scene class interface - which all scenes must inherit
//  and implement. Provides basic camera / animation controls
class Scene {
protected:
    CameraPath cameraPath[SCENE_MAX_CAMERAPATHS];
    ParamPath paramPath[SCENE_MAX_CAMERAPATHS];
    D3DXVECTOR3 endTimePath[SCENE_MAX_CAMERAPATHS];
    int numAnimPaths;

    int animFrame;
    double animFrameTime;
    double animFrameLength;

public:

    // ---> Scene interface parameters
    double time;
    double FOV;
    double BPM;
    float bloomAmount;
    float bloomThreshold;
    float blurAmount;
    float saturate;
    bool debugMode;
    bool isLoaded;

    // ---> Scene interface functions
    Scene();

    //! The load scene function - must be implemented
    virtual void load();
    //! The render scene function - must be implemented
    virtual void render( ShaderGlobalSemantics &shaderSem, D3DXMATRIX &matView, D3DXMATRIX &matProjection );
    //! The update scene function - must be implemented
    virtual void update( D3DXMATRIX &matView, ShaderGlobalSemantics *shaderSem );
    //! The release scene function - must be implemented
    virtual void release();

    //! The add parameter vector function - optional, Defaults to no parameters
    virtual void addParamVector( ParamPath *dest );
    //! The update parameter vector function - optional, Defaults to no parameters
    virtual void updateParamVector( ParamPath &paramPath, double pathTime );

    // ---> Animation functions

    //! Loads animation paths from file
    void loadSceneAnimationPaths( const char* fileName );
    //! Gets the current animation frame's number length and time
    void getSceneAnimationFrame( int *animFrame, double *animFrameTime, double *animFrameLength );
};

#endif
