/** \file SceneAbstract2.h
<h2> Abstract Scenes 2 </h2> <br>
Copyright (C) 2011 Xi Chen <br>

-------------------------------------------------------------------------<br>
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version. <br>

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details. <br>

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>. <br>
-------------------------------------------------------------------------<br>

\brief Abstract Scenes 2
\author Xi Chen hypernewbie@gmail.com
\date 15/11/2011

*/

#ifndef __SCENE_ABSTRACT2_H__
#define __SCENE_ABSTRACT2_H__

#include "Scene.h"

class SceneAbstract2 : public Scene {
    TextureArray textures;

    // --> Torus Scene
    dxPrimitivesObj *torus;
    LPDIRECT3DCUBETEXTURE9  torusCubeMap;
    Shader torusShader;
    dxPrimitivesObj *torusSkyDome;

    void renderTorus(ShaderGlobalSemantics &shaderSem, D3DXMATRIX &matView, D3DXMATRIX &matProjection);

    // --> Julia Scene
    dxPrimitivesObj *juliaSphere;
    Shader juliaShader;
    D3DXVECTOR4 juliaC;
    LPDIRECT3DCUBETEXTURE9  juliaIrradianceMap;
    LPDIRECT3DCUBETEXTURE9  juliaReflectanceMap;

    void renderJulia(ShaderGlobalSemantics &shaderSem, D3DXMATRIX &matView, D3DXMATRIX &matProjection);

    // --> Sphere Scene
    float sphereExtrude;
    dxPrimitivesObj *sphereFront;
    dxPrimitivesObj *sphereBack;
    Shader sphereShader;
    LPDIRECT3DCUBETEXTURE9  sphereCubeMap;

    void renderSphere(ShaderGlobalSemantics &shaderSem, D3DXMATRIX &matView, D3DXMATRIX &matProjection);

    // --> Warp Scene
    Shader warpShader;

    void renderWarp(ShaderGlobalSemantics &shaderSem, D3DXMATRIX &matView, D3DXMATRIX &matProjection);

public:
    void load();
    void render(ShaderGlobalSemantics &shaderSem, D3DXMATRIX &matView, D3DXMATRIX &matProjection);
    void update( D3DXMATRIX &matView, ShaderGlobalSemantics *shaderSem );
    void release();

    void addParamVector( ParamPath *dest );
    void updateParamVector( ParamPath &paramPath, double pathTime );
};

#endif