/** \file SceneChrono.h
<h2> Chronosphere Scene </h2> <br>
Copyright (C) 2011 Xi Chen <br>

-------------------------------------------------------------------------<br>
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version. <br>

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details. <br>

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>. <br>
-------------------------------------------------------------------------<br>

\brief Chronosphere Scene
\author Xi Chen hypernewbie@gmail.com
\date 15/11/2011

*/

#ifndef __SCENE_CHRONO_SPHERE_H__
#define __SCENE_CHRONO_SPHERE_H__

#include "Scene.h"

#define SCENECHRONO_SHADER_FILE "ChronoSphere/chronoSphere.fx"
#define SCENECHRONO_VOLUME_FILE "ChronoSphere/cytovirus.dds"
#define SCENECHRONO_CUBEMAP_FILE "ChronoSphere/reflection.dds"
#define SCENECHRONO_NOISE_FILE "ChronoSphere/reflection.dds"
#define SCENECHRONO_RINGSHADER_FILE "ChronoSphere/chronoRing.fx"
#define SCENECHRONO_RINGMODEL_FILE "ChronoSphere/ring1.3ds"
#define SCENECHRONO_RINGMODELTEX_FILE "ChronoSphere/ring.png"
#define SCENECHRONO_RINGMODELTEX2_FILE "ChronoSphere/ringSpecular.jpg"
#define SCENECHRONO_RINGMODELTEX3_FILE "ChronoSphere/ringLightExtemption.jpg"
#define SCENECHRONO_RINGMODELTEX4_FILE "ChronoSphere/ringOcclusion.jpg"
#define SCENECHRONO_ROOMMODEL_FILE "ChronoSphere/generatorRoom.obj"
#define SCENECHRONO_ROOMTEX_FILE "ChronoSphere/roomOcclusion.png"
#define SCENECHRONO_ROOMSHADER_FILE "ChronoSphere/chronoRoom.fx"
#define SCENECHRONO_TUBETEX0_FILE "ChronoSphere/screen1.jpg"
#define SCENECHRONO_TUBETEX1_FILE "ChronoSphere/screen2.jpg"
#define SCENECHRONO_TUBETEX2_FILE "ChronoSphere/screen2shine.jpg"

#define SCENECHRONO_RINGS 3

class SceneChrono : public Scene {
    TextureArray chronoTextures;

    Shader chronoSphereShader;
    dxPrimitivesObj *sphere;

    LPDIRECT3DVOLUMETEXTURE9 cytoVirusVolume;
    LPDIRECT3DCUBETEXTURE9  cytoVirusCubeMap;

    Shader chronoRingShader;
    Model ring;
    D3DXVECTOR3 ringRotation[SCENECHRONO_RINGS];

    Shader chronoRoomShader;
    Model generatorRoom;

    dxPrimitivesObj *tube;

    void renderTube(ShaderGlobalSemantics &shaderSem, 
        D3DXMATRIX &matView, D3DXMATRIX &matProjection, bool reverseOrder);

public:
    void load();
    void render(ShaderGlobalSemantics &shaderSem, D3DXMATRIX &matView, D3DXMATRIX &matProjection);
    void update( D3DXMATRIX &matView, ShaderGlobalSemantics *shaderSem );
    void release();
};

#endif