/** \file SceneSpace.h
<h2> Solar System Scene </h2> <br>
Copyright (C) 2011 Xi Chen <br>

-------------------------------------------------------------------------<br>
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version. <br>

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details. <br>

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>. <br>
-------------------------------------------------------------------------<br>

\brief Solar System Scene
\author Xi Chen hypernewbie@gmail.com
\date 15/11/2011

*/

#ifndef __SCENE_SPACE_H__
#define __SCENE_SPACE_H__

#include "Scene.h"
#include "../GameHeader/Planet.h"

#define SPACE_MAX_PLANETS 10
#define SPACE_MAX_CHILDREN 10
#define SPACE_ROOT_PLANET 0

using namespace std;

struct PlanetState {
    int parent;
    float orbitRadius;
    float angleH;
    float angleV;
    int nChildren;
    int child[SPACE_MAX_CHILDREN];

    D3DXVECTOR3 pos;

    PlanetState();
    void addChild(int c);
};

class SceneSolarSystem : public Scene {
public:
    int numPlanets;
    Texture spaceBackgroundMap; 
    Planet planet[SPACE_MAX_PLANETS];
    PlanetState state[SPACE_MAX_PLANETS];

    Shader atmosShader;

    D3DXVECTOR3 planetPosition( int planetIndex );

    void load();
    void render(ShaderGlobalSemantics &shaderSem, D3DXMATRIX &matView, D3DXMATRIX &matProjection );
    void update( D3DXMATRIX &matView, ShaderGlobalSemantics *shaderSem );
    void release();

protected:
    void renderPlanet(int planetIndex, ShaderGlobalSemantics &shaderSem);
};

#endif