/** \file SceneAbstract2.cpp
<h2> Abstract Scenes 2 </h2> <br>
Copyright (C) 2011 Xi Chen <br>

-------------------------------------------------------------------------<br>
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version. <br>

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details. <br>

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>. <br>
-------------------------------------------------------------------------<br>

\brief Abstract Scenes 2
\author Xi Chen hypernewbie@gmail.com
\date 15/11/2011

*/

#include "../GameHeader/SceneAbstract2.h"
#include "../EngineHeader/DXSound.h"

void SceneAbstract2::load() {
    loadSceneAnimationPaths( "Abstract2/anim.path" );
    // --> Torus Scene
    textures.load(0, "Abstract2/smoke.jpg");
    torus = dxPrimitiveCreateTorus(100, 30, 24, 24);
    
    torusShader.loadFile( "Abstract2/torusShader.fx" );
    torusSkyDome = dxPrimitiveCreateSphereMesh(256, 8, 8);
    D3DXCreateCubeTextureFromFile( d3dDevice, "Abstract2/torusEnv.dds", &torusCubeMap );
    // --> Julia Scene
    juliaSphere = dxPrimitiveCreateSphereMesh( 3.0f, 32, 32 );
    juliaShader.loadFile( "Abstract2/juliaSetShader.fx", false, 0, false );
    D3DXCreateCubeTextureFromFile( d3dDevice, "Abstract2/juliaIrradiance.dds", &juliaIrradianceMap );
    D3DXCreateCubeTextureFromFile( d3dDevice, "Abstract2/juliaReflectance.dds", &juliaReflectanceMap );
    juliaC = D3DXVECTOR4( 0.0, 0.0, 0.0, 0.0 );
    // --> Sphere Scene
    sphereExtrude = 2.0f;
    sphereFront = dxPrimitiveCreateSphereMesh(1, 512, 512);
    sphereBack = dxPrimitiveCreateSphereMesh(256, 32, 32);
    sphereShader.loadFile( "Abstract2/sphereShader.fx" );
    D3DXCreateCubeTextureFromFile( d3dDevice, "Abstract2/sphereEnv.dds", &sphereCubeMap );
    // --> Warp Scene
    textures.load(1, "Abstract2/bricks.png");
    warpShader.loadFile( "Abstract2/warpShader.fx" );
    isLoaded = true;
}

void SceneAbstract2::renderTorus(ShaderGlobalSemantics &shaderSem, 
        D3DXMATRIX &matView, D3DXMATRIX &matProjection) {

    float timeTorus = animFrameTime * 10.0f;

    shaderSem.matView = matView;
    shaderSem.matProj = matProjection;

    bloomAmount = 0.5f; bloomThreshold = 0.6f;

    // --> Update FFT spectrum
    float soundLow, soundMid, soundHigh;
    dxSoundSystemLMH( &soundLow, &soundMid, &soundHigh );
    sphereExtrude = 0.7f + soundLow * 2.4f + soundMid * 0.75f + soundHigh * 0.45f;
    // --> Render Scene
    dxPushMatrix();
        dxScale( 10, 10,10 );
        shaderSem.matWorld = dxGetMatrix();
        dxCommitMatrix();
        torusShader.setGlobalSemantics( &shaderSem );
        torusShader.e()->SetTexture("cubeMapTexture", torusCubeMap); 
        torusShader.e()->SetFloat("skyFactor", sphereExtrude); 
        for(int a = torusShader.begin("Skybox"); a < torusShader.nPasses(); a++) {
            torusShader.pass(a);
                dxSphere( torusSkyDome );
            torusShader.endPass();
        }
        torusShader.end();
    dxPopMatrix();

    dxPushMatrix();
        shaderSem.matWorld = dxGetMatrix();
        dxCommitMatrix();
        torusShader.setGlobalSemantics( &shaderSem );
        torusShader.e()->SetTexture("colourTexture", textures[0]);         dxTexture(0, textures[0]); 
        torusShader.e()->SetTexture("cubeMapTexture", torusCubeMap); 
        torusShader.e()->SetFloat("time", timeTorus); 
        for(int a = torusShader.begin("Torus"); a < torusShader.nPasses(); a++) {
            torusShader.pass(a);
                dxTorus( torus );
            torusShader.endPass();
        }
        torusShader.end();
    dxPopMatrix();
}

void SceneAbstract2::renderJulia(ShaderGlobalSemantics &shaderSem, 
        D3DXMATRIX &matView, D3DXMATRIX &matProjection) {
    
    shaderSem.matView = matView;
    shaderSem.matProj = matProjection;

    bloomAmount = 0.4f; bloomThreshold = 0.8f;

    if( key['Y'] ) juliaC.x -= 0.0015f; if( key['H'] ) juliaC.x += 0.0015f;
    if( key['U'] ) juliaC.y -= 0.0015f; if( key['J'] ) juliaC.y += 0.0015f;
    if( key['I'] ) juliaC.z -= 0.0015f; if( key['K'] ) juliaC.z += 0.0015f;
    if( key['O'] ) juliaC.w -= 0.0015f; if( key['L'] ) juliaC.w += 0.0015f;
    if( key['P'] ) juliaC = D3DXVECTOR4( 0.0f, 0.0f, 0.0f, 0.0f );

    dxPushMatrix();
        dxScale( 10, 10,10 );
        shaderSem.matWorld = dxGetMatrix();
        dxCommitMatrix();
        juliaShader.setGlobalSemantics( &shaderSem );
        juliaShader.e()->SetTexture("irradianceMapTexture", juliaIrradianceMap); 
        for(int a = juliaShader.begin("Skybox"); a < juliaShader.nPasses(); a++) {
            juliaShader.pass(a);
                dxSphere( torusSkyDome );
            juliaShader.endPass();
        }
        juliaShader.end();
    dxPopMatrix();

    dxPushMatrix();
        shaderSem.matWorld = dxGetMatrix();
        dxCommitMatrix();
        juliaShader.setGlobalSemantics( &shaderSem );
        juliaShader.e()->SetTexture("irradianceMapTexture", juliaIrradianceMap); 
        juliaShader.e()->SetTexture("cubeMapTexture", juliaReflectanceMap); 
        juliaShader.e()->SetFloat("Cx", juliaC.x); 
        juliaShader.e()->SetFloat("Cy", juliaC.y); 
        juliaShader.e()->SetFloat("Cz", juliaC.z); 
        juliaShader.e()->SetFloat("Cw", juliaC.w);
        for(int a = juliaShader.begin("JuliaSet"); a < juliaShader.nPasses(); a++) {
            juliaShader.pass(a);
                dxSphere( juliaSphere );
            juliaShader.endPass();
        }
        juliaShader.end();
    dxPopMatrix();
}

void SceneAbstract2::renderSphere(ShaderGlobalSemantics &shaderSem, 
        D3DXMATRIX &matView, D3DXMATRIX &matProjection) {
    // --> Update FFT spectrum
    float soundLow, soundMid, soundHigh;
    dxSoundSystemLMH( &soundLow, &soundMid, &soundHigh );
    sphereExtrude = soundLow * 4.0f + soundMid * 1.0f + soundHigh * 0.5f;
    // --> Render Scene
    shaderSem.matView = matView;
    shaderSem.matProj = matProjection;
    bloomAmount = 0.6f; bloomThreshold = 0.6f;
    dxPushMatrix();
        shaderSem.matWorld = dxGetMatrix();
        dxCommitMatrix();
        sphereShader.setGlobalSemantics( &shaderSem );
        sphereShader.e()->SetTexture("cubeMapTexture", sphereCubeMap); 
        // Background sphere
        for(int a = sphereShader.begin("SkySphere"); a < sphereShader.nPasses(); a++) {
            sphereShader.pass(a);
            dxSphere( sphereBack );
            sphereShader.endPass();
        }
        sphereShader.end();
        // Front sphere
        sphereShader.e()->SetFloat("extrude", sphereExtrude);
        for(int a = sphereShader.begin("FrontSphere"); a < sphereShader.nPasses(); a++) {
            sphereShader.pass(a);
            dxSphere( sphereFront );
            sphereShader.endPass();
        }
        sphereShader.end();
    dxPopMatrix();
}

void SceneAbstract2::renderWarp(ShaderGlobalSemantics &shaderSem, 
        D3DXMATRIX &matView, D3DXMATRIX &matProjection) {
    shaderSem.matView = matView;
    shaderSem.matProj = matProjection;
    dxPushMatrix();
        shaderSem.matWorld = dxGetMatrix();
        dxCommitMatrix();
        warpShader.setGlobalSemantics( &shaderSem );
        warpShader.e()->SetTexture("colourTexture", textures[1]); 
        warpShader.e()->SetFloat("aspect", (float)d3dWindowHeight / (float)d3dWindowWidth ); 
        warpShader.e()->SetFloat("time", time * 2.0f );
        for(int a = warpShader.begin("Warp"); a < warpShader.nPasses(); a++) {
            warpShader.pass(a);
            dxBackgroundPicture(d3dWindowWidth, d3dWindowHeight);
            warpShader.endPass();
        }
        warpShader.end();
    dxPopMatrix();
}

void SceneAbstract2::render(ShaderGlobalSemantics &shaderSem, 
        D3DXMATRIX &matView, D3DXMATRIX &matProjection) {
    if( debugMode ) {
        FOV = 75;
        renderWarp( shaderSem, matView, matProjection ); 
        return;
    }
    bloomAmount = 0.5f; bloomThreshold = 0.7f;
    switch( animFrame ) {
        case 0: renderSphere( shaderSem, matView, matProjection ); break;
        case 1: renderTorus( shaderSem, matView, matProjection ); break;
        case 2: renderWarp( shaderSem, matView, matProjection ); break;
        case 3: renderJulia( shaderSem, matView, matProjection ); break;
    }
}

void SceneAbstract2::update( D3DXMATRIX &matView, ShaderGlobalSemantics *shaderSem ) {
    getSceneAnimationFrame( &animFrame, &animFrameTime, &animFrameLength );
    if( animFrameLength < 0.000001 ) animFrameLength = 0.000001;
    FOV = 90;
    double cameraPathTimeFactor = animFrameTime / animFrameLength;
    switch( animFrame ) {
        case 0: FOV = 155; break;
        case 1: FOV = 140; break;
        case 2: FOV = 140; break;
        case 3: FOV = 75; cameraPathTimeFactor /= 0.8; break;
    }
    cameraPath[ animFrame ].setCamera( cameraPathTimeFactor * 0.8, matView, shaderSem );
    updateParamVector( paramPath[ animFrame ], cameraPathTimeFactor * 0.8 );
}

void SceneAbstract2::addParamVector( ParamPath *dest ) {
    if( !dest ) return;
    D3DXVECTOR3 paramVector[2] = {
        D3DXVECTOR3( juliaC.x, juliaC.y, juliaC.z ),
        D3DXVECTOR3( juliaC.w, 0.0f, 0.0f )
    };
    dest->addFrame( paramVector, 2 );
}

void SceneAbstract2::updateParamVector( ParamPath &paramPath, double pathTime ) {
    paramPath.currentTime = pathTime;
    juliaC.x = paramPath[0].x; juliaC.y = paramPath[0].y;
    juliaC.z = paramPath[0].z; juliaC.w = paramPath[1].x;
}

void SceneAbstract2::release() {
    textures.clear();
    torus->release();
    torusCubeMap->Release();
    torusShader.release();
    torusSkyDome->release();
    juliaSphere->release();
    juliaShader.release();
    juliaIrradianceMap->Release();
    juliaReflectanceMap->Release();
    sphereFront->release();
    sphereBack->release();
    sphereShader.release();
    sphereCubeMap->Release();
    warpShader.release();
    isLoaded = false;
}