/** \file SceneTunnel.cpp
<h2> Cube Tunnel Scene </h2> <br>
Copyright (C) 2011 Xi Chen <br>

-------------------------------------------------------------------------<br>
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version. <br>

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details. <br>

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>. <br>
-------------------------------------------------------------------------<br>

\brief Cube Tunnel Scene
\author Xi Chen hypernewbie@gmail.com
\date 15/11/2011

*/

#include "../GameHeader/SceneCubeTunnel.h"
#include "../EngineHeader/DXSound.h"

float sceneCubeTunnel_randfloat(float min, float max) {
    return min + (max-min) * (float(rand()) / float(RAND_MAX));
}  

void SceneCubeTunnel::load() {
    loadSceneAnimationPaths( "CubeTunnel/anim.path" );
    cubeTunnelShader.loadFile( SCENE_CUBETUNNEL_SHADERFILE );
    ringShader.loadFile( SCENE_CUBETUNNEL_RINGSHADERFILE );
    ring = dxPrimitiveCreateRing( 1.0 , 0.5 );
    cubeTunnelTextures.load( 0, SCENE_CUBETUNNEL_RINGTEXTURE );
    cubeTunnelTextures.load( 1, SCENE_CUBETUNNEL_RINGTEXTURE2 );
    isLoaded = true;
}

void SceneCubeTunnel::render(ShaderGlobalSemantics &shaderSem, 
    D3DXMATRIX &matView, D3DXMATRIX &matProjection) {

    FOV = 110;

    shaderSem.matView = matView;
    shaderSem.matProj = matProjection;
    sgs = &shaderSem;

    float t1 = (time + 0.0f) * 0.01 * 1550.0f, 
          t2 = (time + 1.0f) * 0.0261 * 1550.0f,
          t3 = (time + 4.0f) * 0.0061 * 1550.0f,
          t4 = (time + 5.0f) * 0.0131 * 1550.0f,
          t5 = (time +12.0f) * 0.0221 * 1550.0f,
          t6 = (time +23.0f) * 0.0101 * 1550.0f;

    float soundLow, soundMid, soundHigh;
    dxSoundSystemLMH( &soundLow, &soundMid, &soundHigh );
    soundLow = max( soundLow - 0.3f, 0 );

    dxPushMatrix();
        dxScale( 1000.0, 1000.0, 1000.0 );
        sgs->matWorld = dxGetMatrix();
        cubeTunnelShader.setGlobalSemantics( sgs );
        dxCommitMatrix();
        for(int a = cubeTunnelShader.begin("Background"); a < cubeTunnelShader.nPasses(); a++) {
            cubeTunnelShader.pass(a);
            dxCube();
            cubeTunnelShader.endPass();
        }
        cubeTunnelShader.end();
    dxPopMatrix();

    dxPushMatrix();

        currentTechnique = "InnerRings";
        renderTunnel( 0, 80, -320.0f, 320.0f, 20.0f, t1 );
        renderTunnel( 23, 160, -320.0f, 320.0f, 20.0f, t2 );
        renderTunnel(123, 200, -320.0f, 320.0f, 20.0f, t3 );

        currentTechnique = "AlphaRings";
        renderTunnel( 12, 160, -320.0f, 320.0f, 40.0f, t2 );
        renderTunnel( 22, 160, -320.0f, 320.0f, 80.0f, t1 );

        currentTechnique = "InnerRings";
        renderTunnel( 24, 100, -320.0f, 320.0f, 160.0f, t4, 16.0, 2.0, 4.0 );

        currentTechnique = "OuterRings";
        renderTunnel( 24, 35, -640.0f, 640.0f, 320.0f, t5, 64.0, 5.0, 16.0 );
        renderTunnel( 24, 50, -640.0f, 640.0f, 400.0f, t6, 64.0, 5.0, 32.0 );
        renderTunnel( 24, 70, -640.0f, 640.0f, 480.0f, t1, 96.0, 8.0, 32.0 );

    dxPopMatrix();

    dxPushMatrix();

        dxPushMatrix();
            dxTranslate(-640.0f, 0, 0);
            dxRotateY( 90 );
            dxScale( 320.0, 320.0, 320.0 );
            sgs->matWorld = dxGetMatrix();
            ringShader.setGlobalSemantics( sgs );
            dxCommitMatrix();

            dxTexture( 0, cubeTunnelTextures[1]); 
            ringShader.e()->SetTexture( "ringTexture", cubeTunnelTextures[1] );
            ringShader.e()->SetFloat("visibility", 1.5 + soundLow * 4.5f);
            ringShader.e()->SetVector("colour", &D3DXVECTOR4(0.7f, 1.0f, 0.7f, 1.0f));

            for(int a = ringShader.begin("GlowRings"); a < ringShader.nPasses(); a++) {
                ringShader.pass(a);
                dxRing( ring );
                ringShader.endPass();
            }
            ringShader.end();

        dxPopMatrix();

        for( float x = -640.0f; x < 640.0f; x+=128.0f ) {
            dxPushMatrix();
                dxTranslate(x, 0, 0);
                dxRotateY( 90 );
                dxScale( 80.0, 80.0, 80.0 );
                sgs->matWorld = dxGetMatrix();
                ringShader.setGlobalSemantics( sgs );
                dxCommitMatrix();

                dxTexture( 0, cubeTunnelTextures[0]); 
                ringShader.e()->SetTexture( "ringTexture", cubeTunnelTextures[0] );
                ringShader.e()->SetFloat("visibility", 0.05 + soundLow * 0.75f);
                ringShader.e()->SetVector("colour", &D3DXVECTOR4(0.8f, 1.0f, 0.8f, 1.0f));

                for(int a = ringShader.begin("GlowRings"); a < ringShader.nPasses(); a++) {
                    ringShader.pass(a);
                    dxRing( ring );
                    ringShader.endPass();
                }
                ringShader.end();

            dxPopMatrix();
        }
        for( float x = -480.0f; x < 480.0f; x+=8.0f ) {
            dxPushMatrix();
                dxTranslate(x, 0, 0);
                dxRotateY( 90 );
                dxScale( 3.0, 3.0, 3.0 );
                sgs->matWorld = dxGetMatrix();
                ringShader.setGlobalSemantics( sgs );
                dxCommitMatrix();

                dxTexture( 0, cubeTunnelTextures[0]); 
                ringShader.e()->SetTexture( "ringTexture", cubeTunnelTextures[0] );
                ringShader.e()->SetFloat("visibility", 0.2);
                ringShader.e()->SetVector("colour", &D3DXVECTOR4(0.5f, 1.0f, 0.5f, 1.0f));

                for(int a = ringShader.begin("GlowRings"); a < ringShader.nPasses(); a++) {
                    ringShader.pass(a);
                    dxRing( ring );
                    ringShader.endPass();
                }
                ringShader.end();

            dxPopMatrix();
        }

        
    dxPopMatrix();

    dxCommitMatrix();
}

void SceneCubeTunnel::update( D3DXMATRIX &matView, ShaderGlobalSemantics *shaderSem ) {
    getSceneAnimationFrame( &animFrame, &animFrameTime, &animFrameLength );
    if( animFrameLength < 0.000001 ) animFrameLength = 0.000001;
    double cameraPathTimeFactor = animFrameTime / animFrameLength;
    cameraPath[ animFrame ].setCamera( cameraPathTimeFactor , matView, shaderSem );
}

void SceneCubeTunnel::renderTunnelCube( float start, float end, float radius, float rot,
                                        float scaleX, float scaleY, float scaleZ ) {
     dxPushMatrix();

        float originPoint = sceneCubeTunnel_randfloat(start, end);
        float rotation = sceneCubeTunnel_randfloat(0, 360);

        dxRotateX( rotation + rot );
        dxTranslate(originPoint, radius, 0);
        dxScale( scaleX, scaleY, scaleZ );

        sgs->matWorld = dxGetMatrix();
        cubeTunnelShader.setGlobalSemantics( sgs );
        dxCommitMatrix();

        for(int a = cubeTunnelShader.begin(currentTechnique); a < cubeTunnelShader.nPasses(); a++) {
            cubeTunnelShader.pass(a);
            dxCube();
            cubeTunnelShader.endPass();
        }
        cubeTunnelShader.end();

     dxPopMatrix();
}

void SceneCubeTunnel::renderTunnel( int seed, int density, float start, float end, float radius, float rot,
                                    float scaleX, float scaleY, float scaleZ ) {
    srand(seed);
    for(int a = 0; a < density; a++) {
        renderTunnelCube( start, end, radius, rot, scaleX, scaleY, scaleZ );
    }
}

void SceneCubeTunnel::release() {
    cubeTunnelShader.release();
    ring->release();
    ringShader.release();
    cubeTunnelTextures.clear();
    isLoaded = false;
}